function transformString(inputStr) {
    let charMap = {};
    const chars = inputStr.split("");
    chars.forEach(c => charMap[c.toLowerCase()] = (charMap[c.toLowerCase()] || 0) + 1);
    return chars.map(c => charMap[c.toLowerCase()] > 1 ? ")" : "(").join("");
}

function task1Test() {
    const cases = {
        "din": "(((",
        "recede": "()()()",
        "Success": ")())())",
        "(( @": "))(("
    }

    Object.entries(cases).forEach(e => {
        [key, value] = e;
        const transformed = transformString(key);
        console.log(`${key} -> ${transformed}, test: ${value === transformed}`);
    });
}

function isValidSolution(matrix) {

    function getSubpart(matrix, size, i, j) {
        const baseI = i * size;
        const baseJ = j * size;
        return Array.from({ length: size * size }, (_, index) => matrix[baseI + Math.floor(index / size)][baseJ + (index % size)])
    }

    function getSubparts(matrix) {
        const partSize = 3;
        const partsInRow = matrix.length / partSize;
        const partsTotal = partsInRow * partsInRow;
        return Array.from({ length: partsTotal }, (_, i) => getSubpart(matrix, partSize, Math.floor(i / partsInRow), i % partsInRow));
    }

    function isRowValid(set) {
        return options.every(o => set.some(re => re === o));
    }

    const options = [1, 2, 3, 4, 5, 6, 7, 8, 9];
    const rowsValid = matrix.every(isRowValid);

    const columns = Array.from({ length: matrix.length }, (_, i) =>
        Array.from({ length: matrix.length }, (_, j) => matrix[j][i]));
    const columnsValid = columns.every(isRowValid);

    const subParts = getSubparts(matrix);
    const subpartsValid = subParts.every(isRowValid);
    return rowsValid && columnsValid && subpartsValid;
}

function task2Test() {
    let valid = [
        [5, 3, 4, 6, 7, 8, 9, 1, 2],
        [6, 7, 2, 1, 9, 5, 3, 4, 8],
        [1, 9, 8, 3, 4, 2, 5, 6, 7],
        [8, 5, 9, 7, 6, 1, 4, 2, 3],
        [4, 2, 6, 8, 5, 3, 7, 9, 1],
        [7, 1, 3, 9, 2, 4, 8, 5, 6],
        [9, 6, 1, 5, 3, 7, 2, 8, 4],
        [2, 8, 7, 4, 1, 9, 6, 3, 5],
        [3, 4, 5, 2, 8, 6, 1, 7, 9]
    ];
    console.log(`valid test: ${isValidSolution(valid) === true}`);

    let invalidByRow = [
        [5, 5, 4, 6, 7, 8, 9, 1, 2],
        [6, 7, 2, 1, 9, 5, 3, 4, 8],
        [1, 9, 8, 3, 4, 2, 5, 6, 7],
        [8, 5, 9, 7, 6, 1, 4, 2, 3],
        [4, 2, 6, 8, 5, 3, 7, 9, 1],
        [7, 1, 3, 9, 2, 4, 8, 5, 6],
        [9, 6, 1, 5, 3, 7, 2, 8, 4],
        [2, 8, 7, 4, 1, 9, 6, 3, 5],
        [3, 4, 5, 2, 8, 6, 1, 7, 9]
    ];
    console.log(`invalid test: ${isValidSolution(invalidByRow) === false}`);

    let invalidByColumn = [
        [5, 3, 4, 6, 7, 8, 9, 1, 2],
        [6, 7, 2, 1, 9, 5, 3, 4, 8],
        [8, 9, 1, 3, 4, 2, 5, 6, 7],
        [8, 5, 9, 7, 6, 1, 4, 2, 3],
        [4, 2, 6, 8, 5, 3, 7, 9, 1],
        [7, 1, 3, 9, 2, 4, 8, 5, 6],
        [9, 6, 1, 5, 3, 7, 2, 8, 4],
        [2, 8, 7, 4, 1, 9, 6, 3, 5],
        [3, 4, 5, 2, 8, 6, 1, 7, 9]
    ];
    console.log(`invalid test: ${isValidSolution(invalidByColumn) === false}`);

    let invalidBySubpart = [
        [1, 2, 3, 4, 5, 6, 7, 8, 9],
        [2, 3, 4, 5, 6, 7, 8, 9, 1],
        [3, 4, 5, 6, 7, 8, 9, 1, 2],
        [4, 5, 6, 7, 8, 9, 1, 2, 3],
        [5, 6, 7, 8, 9, 1, 2, 3, 4],
        [6, 7, 8, 9, 1, 2, 3, 4, 5],
        [7, 8, 9, 1, 2, 3, 4, 5, 6],
        [8, 9, 1, 2, 3, 4, 5, 6, 7],
        [9, 1, 2, 3, 4, 5, 6, 7, 8]
    ];
    console.log(`invalid test: ${isValidSolution(invalidBySubpart) === false}`);

}

task1Test();
task2Test();